<?php
/**
 * Function checks Precoded Clip vitals including compatible theme, required plugins,
 * multisite compatibility, and prevents duplication if active on network.
 * 
 * @author Krolyn Studios
 * @package WP_Clips_Multisite/Includes
 * @subpackage Checks
 * @since 2.0.0
 */


if( ! defined( 'ABSPATH' ) ) exit;


function wp_clips_vitals_check( $themes, $plugins, $multisite, $clipdir ) {

	$notices = array();
	$class = 'notice-error';

	// Check required plugins are active
	if( isset( $plugins ) ) {
		$plugins_wash = array_filter( $plugins );
		if( ! empty( $plugins_wash ) )
			foreach( $plugins as $plugin ) {
				if( ! is_plugin_active( $plugin ) ) {
					$notices[] = 'requires other plugins to be installed and active';
					break;
				}
			}
	}

	// Check theme is compatible
	if( isset( $themes ) ) {
		$themes_wash = array_filter( $themes );
		$current = basename( get_stylesheet_directory() );
		$parent = basename( get_template_directory() );
		if( ! (
			empty( $themes_wash ) ||
			in_array( $current, $themes ) ||
			in_array( $parent, $themes )
		) ) $notices[] = 'is not compatible with the current theme';
	}
	
	// Check Clip is not a duplicate
	if( ! strpos( $clipdir, '/precoded_0/' ) &&
		file_exists( WPCLIPS_PREC . '0/' . basename( $clipdir ) )
	) {
		$notices[] = __( 'is active on the network', 'wp-clips' );
		if( is_network_admin() ) $class = 'notice-info';
	}

	// Check Clip is multisite compatible
	if( $multisite === false )
		$notices[] = 'is not compatible with multisite';

	if( count( $notices ) > 0 ) {

		// Unclip and alert if fail
		$id = wp_clips_id( $clipdir );
		$clip = basename( $clipdir );
		$clipname = preg_split( '/[._]/', $clip );
		rename( $clipdir, WPCLIPS_PREC . $id . '/unclipped/' . $clip );

		foreach( $notices as $notice ) {
			add_action( 'admin_print_footer_scripts', function() use( $clipname, $class, $notice ) {
				$custom = ( $clipname[0] == 'clip' ) ? ' (custom)' : '';
				?><div class="notice <?php echo $class; ?>">
                	<p><strong><?php echo $clipname[0] . $custom; ?></strong><?php 
                		printf( __( ' %s. Refer to Clip %s.', 'wp-clips' ), $notice, $custom ? 'vitals' : 'documentation' );
					?></p>
				</div><?php
			});
		}
		return false;
	}
	return true;
}