<?php
/**
 * Plugin Name: WP Clips Multisite
 * Plugin URI: http://wpclips.net
 * Description: Protect your code customizations with update-safe Clips.
 * Version: 2.1.1
 * Author: Krolyn Studios
 * Author URI: http://www.krolyn.com
 * License: GPLv2
 *
 * Text Domain: wp-clips
 * Domain Path: /languages/
 *
 * @package WP_Clips_Multisite
 * @author Krolyn Studios
 * @license GPLv2
 * @link http://wpclips.net
 * @version 2.1.1
 */


if( ! defined( 'ABSPATH' ) ) exit;


// Check for multisite environment
add_action( 'plugins_loaded', 'wp_clips_multisite_check' );
function wp_clips_multisite_check() {
	if( ! is_multisite() ) {
		deactivate_plugins( plugin_basename( __FILE__ ) );	
		wp_die( sprintf( '<br /><b>%s</b> %s', __( 'WP Clips Multisite', 'wp-clips' ),
			__( 'requires WordPress multisite. Either enable multisite or delete the plugin and install WP Clips.', 'wp-clips' ) ),
			__( 'WP Clips Multisite', 'wp-clips' ), array( 'back_link' => true ) );
	}
}

// Define constants
define( 'WPCLIPS_ROOT', trailingslashit( dirname( __FILE__ ) ) );
define( 'WPCLIPS_VERSION', '2.1.1' );
define( 'WPCLIPS_CLIP', WPCLIPS_ROOT . 'clip_' );
define( 'WPCLIPS_PREC', WPCLIPS_ROOT . 'precoded_' );

// Load textdomain
add_action( 'plugins_loaded', 'wp_clips_load_textdomain' );
function wp_clips_load_textdomain() {
	load_plugin_textdomain( 'wp-clips', false, WPCLIPS_ROOT . 'languages' );
}


/**
 * Return blog ids
 * (network defaults to 0)
 *
 * @since 2.0.0
 */

// Return current blog id
function wp_clips_current_id() {
	global $blog_id;
	$id = is_network_admin() ? 0 : $blog_id;
	return $id;
}

// Return a Clip blog id
function wp_clips_id( $clipdir ) {

	if( strpos( $clipdir, WPCLIPS_PREC ) === false )
		$id = substr( $clipdir, -1 );
	else {
		$dir = strpos( $clipdir, '/unclipped/' ) ? dirname( $clipdir ) : $clipdir;
		$id = substr( dirname( $dir ), -1 );
	}
	return $id;
}


/**
 * Includes
 *
 * @since 1.2.0
 */

// Add vitals check
include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
include_once( WPCLIPS_ROOT . 'includes/checks.php' );

// Add admin Control (if in admin)
if( is_admin() )
	require_once( WPCLIPS_ROOT . 'includes/admin.php' );

// Add uninstalls (if plugin delete)
register_uninstall_hook( __FILE__ , 'wp_clips_uninstall' );
function wp_clips_uninstall() {
	foreach( wp_clips_array( WPCLIPS_ROOT ) as $clipdir ) {
		$uninstall = $clipdir . '/uninstall.php';
		if( file_exists( $uninstall ) )
			include $uninstall;
	}
}


/**
 * Load all active Clips
 *
 * @since 2.0.0
 */

add_action( 'after_setup_theme', 'wp_clips_initialize', 5 );

function wp_clips_initialize() {
	
	global $blog_id;

	// Include core Clip functions
	$file = WPCLIPS_CLIP . '0/core-functions.php';
	if( file_exists( $file ) ) include_once( $file );

	// For network and current blog
	foreach( array( 0, $blog_id ) as $id ) {

		// Load Precoded Clips (if any)
		$clipdirs = WPCLIPS_PREC . $id . '/clip-*/';
		foreach( glob( $clipdirs ) as $clipdir ) {

			// Vitals check before loading
			if( file_exists( $clipdir . 'vitals.php' ) ) {
				$themes = $plugins = $multisite = null;
				include_once( $clipdir . 'vitals.php' );
				if( ! wp_clips_vitals_check( $themes, $plugins, $multisite, $clipdir ) )
					continue;
			}
			include_once( $clipdir . 'clip-functions.php' );
		}

		// Vitals check before loading custom Clip
		$file = WPCLIPS_CLIP . $id . '/vitals.php';
		if( file_exists( $file ) ) {
			$themes = $plugins = $multisite = null;
			include_once( $file );
			$clipdir = WPCLIPS_CLIP . $id;
			if( ! wp_clips_vitals_check( $themes, $plugins, $multisite, $clipdir ) )
				continue;
		}

		// Load custom functions
		$file = WPCLIPS_CLIP . $id . '/custom-functions.php';
		if( file_exists( $file ) ) include_once( $file );

		// Enqueue custom jquery and stylesheet (if exists)
		add_action( 'wp_enqueue_scripts', function() use( $id ) {

			// Load custom jquery
			$file = 'clip_' . $id . '/custom-jquery.js';
			if( file_exists( WPCLIPS_ROOT . $file ) )
				wp_enqueue_script( 'clip-custom-jquery-' . $id, plugins_url( $file, __FILE__ ),
									array( 'jquery' ), WPCLIPS_VERSION, true
				);

			// Load custom styles
			$file = 'clip_' . $id . '/custom-style.css';
			if( file_exists( WPCLIPS_ROOT . $file ) )
				wp_enqueue_style(  'clip-custom-style-' . $id, plugins_url( $file, __FILE__ ),
									array(), WPCLIPS_VERSION
				);

		}, 15 );
	}	
}